/*      $OpenBSD: ex_delete.c,v 1.7 2014/11/12 04:28:41 bentley Exp $   */

/* SPDX-License-Identifier: BSD-3-Clause */

/*
 * Copyright (c) 1992, 1993, 1994
 *      The Regents of the University of California.  All rights reserved.
 * Copyright (c) 1992, 1993, 1994, 1995, 1996
 *      Keith Bostic.  All rights reserved.
 * Copyright (c) 2022-2024 Jeffrey H. Johnson <trnsz@pobox.com>
 *
 * See the LICENSE.md file for redistribution information.
 */

#include <sys/types.h>
#include <sys/queue.h>

#include <bitstring.h>
#include <limits.h>
#include <stdio.h>

#include "../common/common.h"

/*
 * ex_delete: [line [,line]] d[elete] [buffer] [count] [flags]
 *
 *      Delete lines from the file.
 *
 * PUBLIC: int ex_delete(SCR *, EXCMD *);
 */
int
ex_delete(SCR *sp, EXCMD *cmdp)
{
        recno_t lno;

        NEEDFILE(sp, cmdp);

        /*
         * !!!
         * Historically, lines deleted in ex were not placed in the numeric
         * buffers.  We follow historic practice so that we don't overwrite
         * vi buffers accidentally.
         */
        if (cut(sp,
            FL_ISSET(cmdp->iflags, E_C_BUFFER) ? &cmdp->buffer : NULL,
            &cmdp->addr1, &cmdp->addr2, CUT_LINEMODE))
                return (1);

        /* Delete the lines. */
        if (del(sp, &cmdp->addr1, &cmdp->addr2, 1))
                return (1);

        /* Set the cursor to the line after the last line deleted. */
        sp->lno = cmdp->addr1.lno;

        /* Or the last line in the file if deleted to the end of the file. */
        if (db_last(sp, &lno))
                return (1);
        if (sp->lno > lno)
                sp->lno = lno;
        return (0);
}
